/**
  Copyright (C) 2000 Helsinki University of Technology, Inc. All rights Reserved.
  XXX   %Z%%M% version %I% date:%G%
  Component:  ECDSACurve
  Package: fi.hut.tcm.ECImplementation
  File:    ECDSACurve.java
*/

package fi.hut.tcm.tessa.ECImplementation;
import fi.hut.tcm.tessa.ECInterfaces.*;
import java.math.BigInteger;
import java.security.*;
import java.io.*;
import java.security.spec.*;

/**
  ECDSACurve
  
  @doc
  An immutable class that represent an elliptic curve .
  
  @see ECDSAParams
  
  @version 1.0, 30.3.1999
  @author Yki Kortesniemi 
*/
class ECDSACurve
  extends AlgorithmParametersSpi
  implements ECDSAParams, AlgorithmParameterSpec {

  /**
    @doc
    The prime field used, <code>p</code>.
  */
  private BigInteger P;
  
  /**
    @doc
    The generator point, <code>G</code>.
  */
  private ECPoint G;
  
  /**
    @doc
    The order of the generator point, <code>N</code>.
   */
  private BigInteger N;
  
  /**
    @doc
    The cofactor, <code>H</code>.
  */
  private BigInteger H;
  
  /**
    @doc
    The first curve parameter, <code>a</code>.
  */
  private BigInteger A;
  
  /**
    @doc
    The second curve parameter, <code>b</code>.
  */
  private BigInteger B;
  
  /**
    @doc
    The names of the standard curves.
  */
  static private String[] standardCurves = {
  	"ANSI X9.62-1998:prime192v1",
  	"ANSI X9.62-1998:prime239v1",
	"ANSI X9.62-1998:prime256v1"
	};
	
 /**
    @doc
    The strengths of the standard curves.
  */
  static int[] standardStrengths = {
  	192,
  	239,
  	256
  };
 
  
  /**
    @doc
    The name of this curve, if it is a standard curve. 
    An Index to the table of standard curves, otherwise -1.
  */
	private int currentCurve = -1;	

	
  /**
    @doc
    The name of this curve, if it is a standard curve. 
    An Index to the table of standard curves, otherwise -1.
  */
  static ECDSACurve getStandardCurve(byte[] encoded) {
	String name = new String(encoded);
	for (int i = 0; i < standardCurves.length; i++) {
		if (name.compareTo(standardCurves[i]) == 0) {
			return new ECDSACurve(standardStrengths[i]);
		}
	}
	throw new IllegalArgumentException("Unknown curve");
}


  /**
    @doc
	Create a new curve, if you know all the parameters.
  */
  protected void setCurve(BigInteger P, ECPoint G, BigInteger N, BigInteger H, BigInteger A, BigInteger B) {
  	this.P = P;
	this.G = G;
	this.N = N;
	this.H = H;
	this.A = A;
	this.B = B;
	this.currentCurve = -1;
  }

  

  /**
    @doc
	The default constructor. Create a standard curve with a strength of 256 bits.
  */
  protected ECDSACurve() {
	this(256);
  }
  

  
  /**
	@doc
	Create one of the standard curves with the given strength.
  */
  protected ECDSACurve(int strength) {
  	if (strength == 192) {
		this.P = new BigInteger("6277101735386680763835789423207666416083908700390324961279");
		this.N = new BigInteger("6277101735386680763835789423176059013767194773182842284081");
		this.H = BigInteger.ONE;
		this.A = new BigInteger("fffffffffffffffffffffffffffffffefffffffffffffffc", 16);
		this.B = new BigInteger("64210519e59c80e70fa7e9ab72243049feb8deecc146b9b1", 16);
	
		byte[] point= (new BigInteger("03188da80eb03090f67cbf20eb43a18800f4ff0afd82ff1012", 16)).toByteArray();
		this.G = (ECPointImpl) OS2P(point);

		this.currentCurve = 0;
	} else if (strength == 239) {
		this.P = new BigInteger("883423532389192164791648750360308885314476597252960362792450860609699839");
		this.N = new BigInteger("7fffffffffffffffffffffff7fffff9e5e9a9f5d9071fbd1522688909d0b", 16);
		this.H = BigInteger.ONE;
		this.A = new BigInteger("7fffffffffffffffffffffff7fffffffffff8000000000007ffffffffffc", 16);
		this.B = new BigInteger("6b016c3bdcf18941d0d654921475ca71a9db2fb27d1d37796185c2942c0a", 16);
	
		byte[] point= (new BigInteger("020ffa963cdca8816ccc33b8642bedf905c3d358573d3f27fbbd3b3cb9aaaf", 16)).toByteArray();
		this.G = (ECPointImpl) OS2P(point);

		this.currentCurve = 1;
	} else if (strength == 256) {
		this.P = new BigInteger("115792089210356248762697446949407573530086143415290314195533631308867097853951");	
		this.N = new BigInteger("ffffffff00000000ffffffffffffffffbce6faada7179e84f3b9cac2fc632551", 16);
		this.H = BigInteger.ONE;
		this.A = new BigInteger("ffffffff00000001000000000000000000000000fffffffffffffffffffffffc", 16);
		this.B = new BigInteger("5ac635d8aa3a93e7b3ebbd55769886bc651d06b0cc53b0f63bce3c3e27d2604b", 16);
	
		byte[] point= (new BigInteger("036b17d1f2e12c4247f8bce6e563a440f277037d812deb33a0f4a13945d898c296", 16)).toByteArray();
		this.G = (ECPointImpl) OS2P(point);

		this.currentCurve = 2;
	} else {
		throw new IllegalArgumentException("Unsupported strength: "+strength);
	}  	
  }
  

	
	/**
    @doc
    Initializes this parameters object using the parameters 
    specified in <code>paramSpec</code>.
    
    @param paramSpec the parameter specification.
    
    @exception InvalidParameterSpecException if the given parameter
    specification is inappropriate for the initialization of this parameter
    object.
  */
  protected void engineInit(AlgorithmParameterSpec paramSpec)
    throws InvalidParameterSpecException  {
	throw new RuntimeException("XXX Not implemented yet");
  }
  
  
  
  /**
    @doc
    Imports the specified parameters and decodes them
    according to the primary decoding format for parameters.
    The primary decoding format for parameters is ASN.1, if an ASN.1
    specification for this type of parameters exists.
    
    @param params the encoded parameters.
    
    @exception IOException on decoding errors
  */
  protected void engineInit(byte[] params)
    throws IOException  {
	throw new RuntimeException("XXX Not implemented yet");
  }
  
  /**
    @doc
    Imports the parameters from <code>params</code> and
    decodes them according to the specified decoding format.
    If <code>format</code> is null, the
    primary decoding format for parameters is used. The primary decoding
    format is ASN.1, if an ASN.1 specification for these parameters
    exists.
    
    @param params the encoded parameters.
    
    @param format the name of the decoding format.
    
    @exception IOException on decoding errors
  */
  protected void engineInit(byte[] params, String format)
    throws IOException  {
	throw new RuntimeException("XXX Not implemented yet");
  }
  
  /**
    @doc
    Returns a (transparent) specification of this parameters
    object.
    <code>paramSpec</code> identifies the specification class in which 
    the parameters should be returned. It could, for example, be
    <code>DSAParameterSpec.class</code>, to indicate that the
    parameters should be returned in an instance of the 
    <code>DSAParameterSpec</code> class.
    
    @param paramSpec the the specification class in which 
    the parameters should be returned.
    
    @return the parameter specification.
    
    @exception InvalidParameterSpecException if the requested parameter
    specification is inappropriate for this parameter object.
  */
  protected AlgorithmParameterSpec engineGetParameterSpec(Class paramSpec)
    throws InvalidParameterSpecException  {
	return null;
  }
  

  
  /**
    @doc
    Returns the parameters in their primary encoding format.
    The primary encoding format for parameters is ASN.1, if an ASN.1
    specification for this type of parameters exists.
    
    @return the parameters encoded using the specified encoding scheme.
    
    @exception IOException on encoding errors.
  */
  protected byte[] engineGetEncoded()
    throws IOException  {
  	if (currentCurve < 0) {
  		return null;
  	}
	else {
		return getStandardName().getBytes();
	}
  }
  

  
  /**
    @doc
    Returns the parameters encoded in the specified format.
    If <code>format</code> is null, the
    primary encoding format for parameters is used. The primary encoding
    format is ASN.1, if an ASN.1 specification for these parameters
    exists.
    
    @param format the name of the encoding format.
    
    @return the parameters encoded using the specified encoding scheme.
    
    @exception IOException on encoding errors.
  */
  protected byte[] engineGetEncoded(String format)
    throws IOException  {
	return null;
  }
  
  /**
    @doc
    Returns a formatted string describing the parameters.
    
    @return a formatted string describing the parameters.
  */
  protected String engineToString() {
	return getStandardName();
 }
  
  /**
    @doc
    Returns the prime field used, <code>p</code>.
    
    @return the prime field, <code>p</code>. 
  */
  public BigInteger getP() {
	return P;
 }
  
  /**
    @doc
    Returns the generator point, <code>G</code>.
    
    @return the generator point, <code>G</code>. 
  */
  public ECPoint getG() {
	return G;
  }
  
  /**
    @doc
    Returns the order of the generator point, <code>N</code>.
    
    @return the order of the generator point, <code>N</code>. 
  */
  public BigInteger getN() {
	return N;
  }
  
  /**
    @doc
    Returns the cofactor, <code>H</code>.
    
    @return the cofactor, <code>H</code>. 
  */
  public BigInteger getH() {
	return H;
  }
  
  /**
    @doc
    Returns the first curve parameter, <code>a</code>.
    
    @return the first curve parameter, <code>a</code>. 
  */
  public BigInteger getA() {
	return A;
  }
  
  /**
    @doc
    Returns the second curve parameter, <code>b</code>.
    
    @return the second curve parameter, <code>b</code>. 
  */
  public BigInteger getB() {
	return B;
  }
  
  /**
    @doc
    Returns the name of the curve, if it is a standard curve. 
    Otherwise returns <code>null</code>
    
    @return the name of a standard curve, 
    	for other curves: <code>null</code>. 
  */
  public String getStandardName() {
	return standardCurves[currentCurve];
  }
  
  /**
    @doc
    Creates an <code>ECPoint</code> from its octet string representation.
    For more information on octet string representations, see ANSI X9.62-1998
    or IEEE P1363.
    
    @return the <code>ECPoint</code> corresponding to the octet string. 
  */
  public ECPoint OS2P(byte[] OS) {
  	if (OS.length < 1) throw new IllegalArgumentException();
  	
  	BigInteger x;
  	BigInteger y;
  	byte[] tmp;
  	int i;
  	
	switch (OS[0]) {
		case 0: // point at infinity
			return new ECPointImpl(this);
		case 2: // compressed point
		case 3:
			tmp = new byte[OS.length-1];
			System.arraycopy(OS, 1, tmp, 0, OS.length-1);
			x = OS2I(tmp);
			if (OS[0] == 2) {
				y = uncompress(x, 0);
				}
			else {
				y = uncompress(x, 1);
				}			
			break;
		case 4: // uncompressed point
		case 6:
		case 7:
			if ((OS.length - 1) % 2 != 0) {
				throw new IllegalArgumentException();
				}
			int l = (OS.length - 1) / 2;
			tmp = new byte[l];

			System.arraycopy(OS, 1, tmp, 0, l);
			x = OS2I(tmp);

			System.arraycopy(OS, l+1, tmp, 0, l);
			y = OS2I(tmp);
			break;
		default: throw new IllegalArgumentException();
	}
	return new ECPointImpl(this, x, y);
  }
  
  /**
    @doc
    Creates a <code>BigInteger</code> from its octet string representation.
    For more information on octet string representations, see ANSI X9.62-1998
    or IEEE P1363.
    
    @return the <code>BigInteger</code> corresponding to the octet string. 
  */
  public BigInteger OS2I(byte[] OS) {
	return new BigInteger(OS);			
  }
  
    /**
    @doc
    Uncompresses a compressed elliptic curve coordinate representation.
    For more information on octet string representations, see ANSI X9.62-1998
    or IEEE P1363.
    
    @return the <code>BigInteger</code> corresponding to the compressed octet string. 
  */
  public BigInteger uncompress(BigInteger x, int yb) {
	if (x == null) throw new NullPointerException("XXX");

  	BigInteger a = x.pow(3).add(A.multiply(x)).add(B).mod(P);
  	BigInteger b = ECUtilities.modSqrt(a, P);
	if (b == null) throw new IllegalArgumentException();
	if ((b.testBit(0) && yb == 1) || (!b.testBit(0) && yb == 0)){
		return b;
	} 
	else {
		return P.subtract(b);
	}
  }

  
  /**
    @doc
    Converts an <code>ECPoint</code> to its octet string representation.
    For more information on octet string representations, see ANSI X9.62-1998
    or IEEE P1363.
    
    @return the octet string corresponding to the <code>ECPoint</code>. 
  */
  public byte[] P2OS(ECPoint p) {
	if (p == null) throw new NullPointerException("XXX");

	if (p.isInfinity()) {
		byte[] tmp = new byte[1];
		tmp[0] = (byte) '\0';
		return tmp;
	}
  	
	return compress(p);
  }
  
  /**
    @doc
    Converts a <code>BigInteger</code> to its octet string representation.
    For more information on octet string representations, see ANSI X9.62-1998
    or IEEE P1363.
    
    @return the octet string corresponding to the <code>BigInteger</code>. 
  */
  public byte[] I2OS(BigInteger d) {
	return  d.toByteArray();			
  }
  
  /**
    @doc
    Compresses the coordinates of an <code>ECPoint</code> as used in an octet
    string representation.
    For more information on octet string representations, see ANSI X9.62-1998
    or IEEE P1363.
    
    @return the octet string corresponding to the <code>ECPoint</code>. 
  */
  public byte[] compress(ECPoint p) {
	if (p == null) throw new NullPointerException("XXX");

  	BigInteger y = p.getY();
  	BigInteger x = p.getX();
	byte[] tmp = x.toByteArray();
	byte[] result = new byte[tmp.length + 1];
	System.arraycopy(tmp, 0, result, 1, tmp.length);
	if (y.testBit(0)) {
		result[0] = (byte) '\03';
	} 
	else {
		result[0] = (byte) '\02';
	}
	return result;
  }
  
  /**
    @doc
    Returns <code>true</code>, if the curve passes a set of validation tests, 
	otherwise returns <code>false</code>.
    
    @return <code>true</code>, if the curve passes a set of validation tests,
	otherwise <code>false</code>. 
  */
  protected boolean verifyCurve() {
	if (!P.isProbablePrime(256)) {
		return false;
	}
	
	if (A.compareTo(BigInteger.ZERO) < 0 || 
		A.compareTo(P.subtract(BigInteger.ONE)) >= 0) {
		return false;
	}

	if (B.compareTo(BigInteger.ZERO) < 0 || 
		B.compareTo(P.subtract(BigInteger.ONE)) >= 0) {
		return false;
	}

	if (G == null) System.out.println("G is a null pointer");
	if (G.getX() == null) System.out.println("G.getX() is a null pointer");


	if (G.getX().compareTo(BigInteger.ZERO) < 0 || 
		G.getX().compareTo(P.subtract(BigInteger.ONE)) >= 0) {
		return false;
	}

	if (G.getY().compareTo(BigInteger.ZERO) < 0 || 
		G.getY().compareTo(P.subtract(BigInteger.ONE)) >= 0) {
		return false;
	}
	
	if (A.multiply(A).multiply(A).multiply(ECUtilities.FOUR).
		add(B.multiply(B).multiply(new BigInteger("27"))).
		mod(P).compareTo(BigInteger.ZERO) == 0) {
		return false;
	}
	

  	BigInteger a = G.getX().pow(3).add(A.multiply(G.getX())).add(B).mod(P);
  	BigInteger b = ECUtilities.modSqrt(a, P);
	if (b == null) {
		return false;
	}
	
	if (!N.isProbablePrime(256)) {
		return false;
	}

	if (N.compareTo(ECUtilities.modSqrt(P, BigInteger.ONE).multiply(ECUtilities.FOUR)) <= 0) {
		return false;
	}
	
	if (!(((ECPointImpl) G.multiply(N)).isInfinity())) {
		return false;
	}
	
	return true;
  }
  
} // ECDSACurve
