/**
  Copyright (C) 2000 Helsinki University of Technology, Inc. All rights Reserved.
  XXX   %Z%%M% version %I% date:%G%
  Component:  ECDSAKeyFactory
  Package: fi.hut.tcm.ECImplementation
  File:    ECDSAKeyFactory.java
*/

package fi.hut.tcm.tessa.ECImplementation;

import java.math.BigInteger;
import fi.hut.tcm.tessa.ECInterfaces.*;

import java.util.*;
import java.lang.*;
import java.security.Key;
import java.security.PublicKey;
import java.security.PrivateKey;
import java.security.KeyFactorySpi;
import java.security.InvalidKeyException;
import java.security.interfaces.DSAParams;
import java.security.spec.DSAPublicKeySpec;
import java.security.spec.DSAPrivateKeySpec;
import java.security.spec.KeySpec;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.X509EncodedKeySpec;
import java.security.spec.PKCS8EncodedKeySpec;

/**
 * This class implements the ECDSA key factory.
 */

public class ECDSAKeyFactory extends KeyFactorySpi {

    /**
     * Generates a public key object from the provided key specification
     * (key material).
     *
     * @param keySpec the specification (key material) of the public key
     *
     * @return the public key
     *
     * @exception InvalidKeySpecException if the given key specification
     * is inappropriate for this key factory to produce a public key.
     */
    protected PublicKey engineGeneratePublic(KeySpec keySpec)
    throws InvalidKeySpecException {
		if (keySpec instanceof ECDSAPublicKeySpec) {
			ECDSAPublicKeySpec spec = (ECDSAPublicKeySpec)keySpec;
			return new ECDSAPublicKeyImpl(spec.getQ(), spec.getParams());
		} else {
			throw new InvalidKeySpecException ("Inappropriate key specification");
		}
    }

    /**
     * Generates a private key object from the provided key specification
     * (key material).
     *
     * @param keySpec the specification (key material) of the private key
     *
     * @return the private key
     *
     * @exception InvalidKeySpecException if the given key specification
     * is inappropriate for this key factory to produce a private key.
     */
    protected PrivateKey engineGeneratePrivate(KeySpec keySpec)
	 throws InvalidKeySpecException {
		if (keySpec instanceof ECDSAPrivateKeySpec) {
			ECDSAPrivateKeySpec spec = (ECDSAPrivateKeySpec)keySpec;
			return new ECDSAPrivateKeyImpl(spec.getD(), spec.getParams());
	    } else {
			throw new InvalidKeySpecException
				("Inappropriate key specification");
		}
	}

    /**
     * Returns a specification (key material) of the given key object
     * in the requested format.
     *
     * @param key the key 
     *
     * @param keySpec the requested format in which the key material shall be
     * returned
     *
     * @return the underlying key specification (key material) in the
     * requested format
     *
     * @exception InvalidKeySpecException if the requested key specification is
     * inappropriate for the given key, or the given key cannot be processed
     * (e.g., the given key has an unrecognized algorithm or format).
     */
    protected KeySpec engineGetKeySpec(Key key, Class keySpec)
    throws InvalidKeySpecException {
	
	ECDSAParams curve;

	try {

	    if (key instanceof fi.hut.tcm.tessa.ECImplementation.ECDSAPublicKeyImpl) {
		
			// Determine valid key specs
			Class spec = Class.forName
				("fi.hut.tcm.ECImplementation.tessa.ECDSAPublicKeySpec");

			if (spec.isAssignableFrom(keySpec)) {
				ECDSAPublicKeyImpl pubKey = (ECDSAPublicKeyImpl)key;
				return new ECDSAPublicKeySpec(pubKey.getParams(), pubKey.getQ());
			} else {
				throw new InvalidKeySpecException("Inappropriate key specification");
			}
		 
	    } else if (key instanceof fi.hut.tcm.tessa.ECImplementation.ECDSAPrivateKeyImpl) {

			// Determine valid key specs
			Class spec = Class.forName
				("fi.hut.tcm.tessa.ECImplementation.ECDSAPrivateKeySpec");

			if (spec.isAssignableFrom(keySpec)) {
				fi.hut.tcm.tessa.ECImplementation.ECDSAPrivateKeyImpl privKey
				= (fi.hut.tcm.tessa.ECImplementation.ECDSAPrivateKeyImpl)key;
				return new ECDSAPrivateKeySpec(privKey.getParams(), privKey.getD());

			} else {
				throw new InvalidKeySpecException("Inappropriate key specification");
			}

		} else {
			throw new InvalidKeySpecException("Inappropriate key type");
	    }

	} catch (ClassNotFoundException e) {
	    throw new InvalidKeySpecException("Unsupported key specification: " + e.getMessage());
	}
	}
    /**
     * Translates a key object, whose provider may be unknown or potentially
     * untrusted, into a corresponding key object of this key factory.
     *
     * @param key the key whose provider is unknown or untrusted
     *
     * @return the translated key
     *
     * @exception InvalidKeyException if the given key cannot be processed by
     * this key factory.
     */
    protected Key engineTranslateKey(Key key) throws InvalidKeyException {
		throw new RuntimeException("Not Implemented yet");
	}
}