/**
  Copyright (C) 2000 Helsinki University of Technology, Inc. All rights Reserved.
  XXX   %Z%%M% version %I% date:%G%
  Component:  ECDSAKeyPairGenerator
  Package: fi.hut.tcm.ECImplementation
  File:    ECDSAKeyPairGenerator.java
*/

package fi.hut.tcm.tessa.ECImplementation;
import fi.hut.tcm.tessa.ECInterfaces.*;
import java.math.BigInteger;
import java.security.*;
import java.io.*;
import java.security.spec.*;


/**
  ECDSAKeyPairGenerator
  
  Creates ECDSA key pairs.
  
*/
public class ECDSAKeyPairGenerator
  extends KeyPairGeneratorSpi {

  /**
  @doc
  The application specific source of randomness.
  */
  private SecureRandom random;
  
  
  /**
  The elliptic curve used.
  */
  private ECDSAParams curve;
  
  /**
  The default constructor.
  */
  public ECDSAKeyPairGenerator() {
	try {
		random = SecureRandom.getInstance("SHA1PRNG");
	} catch (Exception e) {
		throw new Error();
	}
	
	curve = new ECDSACurve();
  }
  
  
  
  /**
    @doc
    Initializes the key pair generator for a certain strength, using
    the default parameter set.
    
    @param strength the strength of the key. This is an
    algorithm-specific metric, such as modulus length, specified in
    number of bits.
    
    @param random the source of randomness for this generator.
  */
  public void initialize(int strength, SecureRandom random) {
	curve = new ECDSACurve(strength);
  }
  
  
  
  /**
    @doc
    Initializes the key pair generator using the specified parameter
    set and user-provided source of randomness.
    
    <p>This concrete method has been added to this previously-defined
    abstract class. (For backwards compatibility, it cannot be abstract.)
    It may be overridden by a provider to initialize the key pair 
    generator. Such an override
    is expected to throw an InvalidAlgorithmParameterException if
    a parameter is inappropriate for this key pair generator.
    If this method is not overridden, it always throws an
    UnsupportedOperationException.
    
    @param params the parameter set used to generate the keys.
    
    @param random the source of randomness for this generator.
    
    @exception InvalidAlgorithmParameterException if the given parameters
    are inappropriate for this key pair generator.
    
    @since JDK1.2
  */
  public void initialize(AlgorithmParameterSpec params, SecureRandom random)
    throws InvalidAlgorithmParameterException  {
  }
  
  
  
  /**
    @doc
    Generates a key pair. Unless an initialization method is called
    using a KeyPairGenerator interface, algorithm-specific defaults
    will be used. This will generate a new key pair every time it
    is called.
  */
  public KeyPair generateKeyPair() {
	if (curve == null) {
		throw new Error();
	}

	BigInteger d = new BigInteger(curve.getN().bitLength(), random);
	d = d.mod(curve.getN());
	PrivateKey pri = new ECDSAPrivateKeyImpl(d, curve);

	ECPoint Q = curve.getG().multiply(d);
	PublicKey pub = new ECDSAPublicKeyImpl(Q, curve);
	
	return new KeyPair(pub, pri);
  }
  
} // ECDSAKeyPairGenerator
