/**
  Copyright (C) 2000 Helsinki University of Technology, Inc. All rights Reserved.
  XXX   %Z%%M% version %I% date:%G%
  Component:  ECDSAPublicKeyImpl
  Package: fi.hut.tcm.ECImplementation
  File:    ECDSAPublicKeyImpl.java
*/

package fi.hut.tcm.tessa.ECImplementation;
import fi.hut.tcm.tessa.ECInterfaces.*;
import java.math.BigInteger;
import java.security.*;

/**
  ECDSAPublicKeyImpl
  
  immutable
*/
class ECDSAPublicKeyImpl
  extends Object
  implements ECDSAPublicKey, ECDSAKey, PublicKey, Key {

  /**
  The public key, i.e. the ECPoint.
  */
  private ECPoint q;
  
  /**
  The elliptic curve used.
  */
  private ECDSAParams curve;
  
  /**
	The default constructor creates a new public key. 
  */
  protected ECDSAPublicKeyImpl(ECPoint q, ECDSAParams curve) {
	this.q = q;
	this.curve = curve;
  }
  
  /**
	Returns the public key.
  */
  public ECPoint getQ() {
  	return q;
  }
  
  /**
    @doc
    Returns the ECDSA-specific key parameters. These parameters are
    never secret.
    
    @return the ECDSA-specific key parameters.
    
    @see ECDSAParams
  */
  public final ECDSAParams getParams() {
  	return curve;
  }
  
  /**
    @doc
    Returns the standard algorithm name for this key. For
    example, "ECDSA" would indicate that this key is a ECDSA key.
    See Appendix A in the <a href=
    "../../../guide/security/CryptoSpec.html#AppA">
    Java Cryptography Architecture API Specification &amp; Reference </a>
    for information about standard algorithm names.
    
    @return the name of the algorithm associated with this key.
  */
  public final String getAlgorithm() {
  	return "ECDSA";
  }
  
  /**
    @doc
    Returns the name of the primary encoding format of this key,
    or null if this key does not support encoding.
    The primary encoding format is
    named in terms of the appropriate ASN.1 data format, if an
    ASN.1 specification for this key exists.
    For example, the name of the ASN.1 data format for public
    keys is <I>SubjectPublicKeyInfo</I>, as
    defined by the X.509 standard; in this case, the returned format is
    <code>"X.509"</code>. Similarly,
    the name of the ASN.1 data format for private keys is
    <I>PrivateKeyInfo</I>,
    as defined by the PKCS #8 standard; in this case, the returned format 
    is
    <code>"PKCS#8"</code>.
    
    @return the primary encoding format of the key.
  */
  public final String getFormat() {
  
  // This is a non-standard solution; the standards are still under development
	return "fi.hut.tcm.ECDSAKey";
  
  }
  
  /**
    @doc
    Returns the key in its primary encoding format, or null
    if this key does not support encoding.
    
    @return the encoded key, or null if the key does not support
    encoding.
  */
  public final byte[] getEncoded() {
  
  // This is a non-standard solution; the standards are still under development
	ECDSACurve curveImpl = (ECDSACurve) curve; 
	byte[] c = null;
	try {
		c = curveImpl.engineGetEncoded();
	}
	catch (java.io.IOException e) {
	}

	byte[] k = curveImpl.P2OS(q);
	byte l = (byte) c.length;
	byte[] result = new byte[c.length + k.length + 1];
	result[0] = l;
	System.arraycopy(c, 0, result, 1, c.length);
	System.arraycopy(k, 0, result, 1 + c.length, k.length);
	return result;
  }
  
} // ECDSAPublicKeyImpl
