/**
  Copyright (C) 2000 Helsinki University of Technology, Inc. All rights Reserved.
  XXX   %Z%%M% version %I% date:%G%
  Component:  ECDSASignature
  Package: fi.hut.tcm.ECImplementation
  File:    ECDSASignature.java
*/

package fi.hut.tcm.tessa.ECImplementation;
import fi.hut.tcm.tessa.ECInterfaces.*;
import java.math.BigInteger;
import java.security.*;
import java.io.*;
import java.security.spec.*;
import java.util.*;

/**
  ECDSASignature
  
  Creates and vefies ECDSA (Elliptic Curve Digital Signature Algorithm) 
  signatures. ECDSA has been defined in ANSI X9.62 and IEEE P1363 standards.
  
  
*/
public class ECDSASignature
  extends SignatureSpi {

  /**
    @doc
    Application-specified source of randomness. 
  */
  protected SecureRandom appRandom;
  
  /**
    @doc
    The private key used for signing.
  */
  private ECDSAPrivateKey privateKey;
  
  /**
    @doc
    The public key used for verifying the signatures.
  */
  private ECDSAPublicKey publicKey;
  
  /**
    @doc
    The message digest used.
  */
  private MessageDigest digest;
  
  /**
  The default constructor that initialises the message digest used.
  */
  public ECDSASignature() {
	try {
		digest = MessageDigest.getInstance("SHA-1");
	} catch (NoSuchAlgorithmException e) {
		throw new RuntimeException("Unable to obtain a message digest!");
	}
  }
  
  
  /**
    @doc
    Initializes this signature object with the specified
    public key for verification operations.
    
    @param publicKey the public key of the identity whose signature is
    going to be verified.
    
    @exception InvalidKeyException if the key is improperly
    encoded, parameters are missing, and so on.  
  */
  protected void engineInitVerify(PublicKey publicKey)
    throws InvalidKeyException  {
	if (! (publicKey instanceof ECDSAPublicKey)) {
		throw new InvalidKeyException("Did not get an ECDSAPublicKey");
	}

	// Can only either Sign or Verify
	privateKey = null;

	this.publicKey = (ECDSAPublicKey) publicKey;

	// The public key must not be infinity
	if (this.publicKey.getQ().isInfinity()) {
		throw new InvalidKeyException("An ECDSAPublicKey can not be Infinity");
	}
	
	// but nQ must equal infinity
	if (!(this.publicKey.getQ().multiply(
			this.publicKey.getQ().getParams().getN()).isInfinity())) {
		throw new InvalidKeyException("The public key does not qualify: nQ != Infinity");
	}

	// And reset the digest
	digest.reset();
  }
  
  
  
  /**
    @doc
    Initializes this signature object with the specified
    private key for signing operations.
    
    @param privateKey the private key of the identity whose signature
    will be generated.
    
    @exception InvalidKeyException if the key is improperly
    encoded, parameters are missing, and so on. 
  */
  protected void engineInitSign(PrivateKey privateKey)
    throws InvalidKeyException  {
	try {
		engineInitSign(privateKey, SecureRandom.getInstance("SHA1PRNG"));
	} catch (NoSuchAlgorithmException e) {
		throw new RuntimeException("Unable to init Signing");
	}
  }
  
  
  
  /**
    @doc
    Initializes this signature object with the specified
    private key for signing operations and with the specified source of
    randomness.
    
    @param privateKey the private key of the identity whose signature
    will be generated.
    
    @exception InvalidKeyException if the key is improperly
    encoded, parameters are missing, and so on. 
  */
  protected void engineInitSign(PrivateKey privateKey, SecureRandom random)
    throws InvalidKeyException  {
	if (! (privateKey instanceof ECDSAPrivateKey)) {
		throw new InvalidKeyException("Did not get an ECDSAPrivateKey!");
	}

	// Can only either Sign or Verify
	publicKey = null;

	this.privateKey = (ECDSAPrivateKey) privateKey;
	
	// And reset the digest
	digest.reset();

	appRandom = random;
  }
  
  
  
  /**
    @doc
    Updates the data to be signed or verified
    using the specified byte.
    
    @param b the byte to use for the update.
    
    @exception SignatureException if the engine is not initialized
    properly.
  */
  protected void engineUpdate(byte b)
    throws SignatureException  {
	// Make sure engine has been initiated
	if (privateKey == null && publicKey == null) {
		throw new SignatureException("ECDSASignature has not been initialised");
	}
	
	digest.update(b);
  }
  
  
  
  /**
    @doc
    Updates the data to be signed or verified, using the 
    specified array of bytes, starting at the specified offset.
    
    @param data the array of bytes.  
    @param off the offset to start from in the array of bytes.  
    @param len the number of bytes to use, starting at offset.
    
    @exception SignatureException if the engine is not initialized 
    properly.
  */
  protected void engineUpdate(byte[] b, int off, int len)
    throws SignatureException  {
	// Make sure engine has been initiated
	if (privateKey == null && publicKey == null) {
		throw new SignatureException("ECDSASignature has not been initialised");
	}

	// Make sure the arguments are legal
	if (len < 1 || off < 0 || len + off > b.length) {
		throw new InvalidParameterException();
	}

	digest.update(b, off, len);
  }
  
  
  
  /**
    @doc
    Returns the signature bytes of all the data
    updated so far.    
    The format of the signature depends on the underlying 
    signature scheme.
    
    @return the signature bytes of the signing operation's result.
    
    @exception SignatureException if the engine is not
    initialized properly.  
  */
  protected byte[] engineSign()
    throws SignatureException  {
	// This algorithm is from ANSI X9.62

	// Make sure engine has been initiated
	if (privateKey == null) {
		throw new SignatureException("ECDSASignature has not been initiated for signing");
	}

	// Calculate the digest
	final BigInteger e = new BigInteger(digest.digest());

	// Then, the elliptic curve and modular computation
	final ECDSAParams curve = privateKey.getParams();
	
	BigInteger k;
	BigInteger r;
	BigInteger s;
	do {
		do {
			k = new BigInteger(curve.getN().bitLength(), appRandom);
			k = k.mod(curve.getN());

			ECPointImpl p = (ECPointImpl) curve.getG().multiply(k);

			BigInteger x1 = p.getX();
			r = x1.mod(curve.getN());
		} while (r.compareTo(BigInteger.ZERO) == 0);
		// s = k^-1 * (e+dr) mod n
		s = k.modInverse(curve.getN()).multiply(e.add(r.multiply(privateKey.getD()))).mod(curve.getN());
	} while (s.compareTo(BigInteger.ZERO) == 0);
	
	// Prepare the signature
	final int length = curve.getN().bitLength() / 8 + 1;

	final byte[] u = r.toByteArray();
	final byte[] v = s.toByteArray();

	byte[] tmp = new byte[2 * length];

	// Then, add r and s
	int diff = length - u.length;
	System.arraycopy(u, 0, tmp, diff, u.length);
		
	diff = length - v.length;
	System.arraycopy(v, 0, tmp, length+diff, v.length);

	return tmp;
  }
 
 
  
  /**
    @doc
    Verifies the passed-in signature.   
    
    @param sigBytes the signature bytes to be verified.
    
    @return true if the signature was verified, false if not. 
    
    @exception SignatureException if the engine is not initialized 
    properly, or the passed-in signature is improperly encoded or 
    of the wrong type, etc.  
  */
  protected boolean engineVerify(byte[] sigBytes)
    throws SignatureException  {
	// Make sure engine has been initiated
	if (publicKey == null) {
		throw new SignatureException();
	}

	final ECDSAParams curve = publicKey.getParams();
	
	final int length = curve.getN().bitLength() / 8 + 1;

	// Signature was too short?
	if (sigBytes.length != 2 * length) {
		return false;
	}

	// Calculate the digest
	final BigInteger e = new BigInteger(digest.digest());


	// Now, extract r and s
	BigInteger r;
	byte[] tmp = new byte[length];
	System.arraycopy(sigBytes, 0, tmp, 0, length);
	r = new BigInteger(tmp);

	BigInteger s;
	System.arraycopy(sigBytes, length, tmp, 0, length);
	s = new BigInteger(tmp);

	// Validate r and s
	if (curve.getN().compareTo(r) <= 0 ||curve.getN().compareTo(s) <= 0) {
		return false;
	}
	
	// Now, the modular and elliptic curve computations
	final BigInteger c = s.modInverse(curve.getN());
	final BigInteger u1 = e.multiply(c).mod(curve.getN());
	final BigInteger u2 = r.multiply(c).mod(curve.getN());

	ECPointImpl p = (ECPointImpl) curve.getG().multiply(u1).add(publicKey.getQ().multiply(u2));
	if (p.isInfinity()) {
		return false;
	}
		
	BigInteger v = p.getX().mod(curve.getN());

	return r.compareTo(v) == 0;
  }
  
 
 
  /**
    @doc
    Sets the specified algorithm parameter to the specified
    value. This method supplies a general-purpose mechanism through
    which it is possible to set the various parameters of this object. 
    A parameter may be any settable parameter for the algorithm, such as 
    a parameter size, or a source of random bits for signature generation 
    (if appropriate), or an indication of whether or not to perform
    a specific but optional computation. A uniform algorithm-specific 
    naming scheme for each parameter is desirable but left unspecified 
    at this time.
    
    @param param the string identifier of the parameter.
    
    @param value the parameter value.
    
    @exception InvalidParameterException if <code>param</code> is an
    invalid parameter for this signature algorithm engine,
    the parameter is already set
    and cannot be set again, a security exception occurs, and so on. 
    
    @deprecated Replaced by <a href =
    "#engineSetParameter(java.security.spec.AlgorithmParameterSpec)">
    engineSetParameter</a>.
  */
  protected void engineSetParameter(String param, Object value)
    throws InvalidParameterException  {
	throw new UnsupportedOperationException();
  }
  
 
 
  /**
  @doc
  Sets the algorithm specific parameters to the values specified by the
  AlgorithmParameterSpec.
  */
  protected void engineSetParameter(AlgorithmParameterSpec params) {
  }
  
  
  
  /**
    @doc
    Gets the value of the specified algorithm parameter. 
    This method supplies a general-purpose mechanism through which it 
    is possible to get the various parameters of this object. A parameter
    may be any settable parameter for the algorithm, such as a parameter 
    size, or  a source of random bits for signature generation (if 
    appropriate), or an indication of whether or not to perform a 
    specific but optional computation. A uniform algorithm-specific 
    naming scheme for each parameter is desirable but left unspecified 
    at this time.
    
    @param param the string name of the parameter.
    
    @return the object that represents the parameter value, or null if
    there is none.
    
    @exception InvalidParameterException if <code>param</code> is an 
    invalid parameter for this engine, or another exception occurs while
    trying to get this parameter.
    
    @deprecated
  */
  protected Object engineGetParameter(String param)
    throws InvalidParameterException  {
	return null;
  }
  
  
  
  /**
  Clones the signature object.
  
  NOT SUPPORTED.
  
  */
  public Object clone()
    throws CloneNotSupportedException  {
	throw new CloneNotSupportedException();
  }
  
} // ECDSASignature
