/**
  Copyright (C) 2000 Helsinki University of Technology, Inc. All rights Reserved.
  XXX   %Z%%M% version %I% date:%G%
  Component:  ECPointImpl
  Package: fi.hut.tcm.ECImplementation
  File:    ECPointImpl.java
*/

package fi.hut.tcm.tessa.ECImplementation;

import fi.hut.tcm.tessa.ECInterfaces.*;
import java.math.BigInteger;


/**
  @doc
  ECPointImpl implements the ECPoint interface.
  
  Immutable.
*/
class ECPointImpl
  extends Object
  implements ECPoint {

  /**
  */
  static BigInteger TWO = new BigInteger("2");;
  
  /**
  */
  static BigInteger THREE = new BigInteger("3");;
  
  /**
  The X-coordinate of the point.
  */
  private BigInteger X;
  
  /**
  The Y-coordinate of the point.
  */
  private BigInteger Y;
  
  /**
  The point is at infinity?
  */
  private boolean isInfinity;
  
  /**
  The elliptic curve
  */
  private ECDSAParams curve;
  
  /**
  */
  ECPointImpl(ECDSAParams curve, BigInteger X, BigInteger Y) {
  	this.X = X;
	this.Y = Y;
	this.curve = curve;
	isInfinity = false;
  }
  


  /**
    Creates a point at infinity
    
  */
  ECPointImpl(ECDSAParams curve) {
  this.curve = curve;
  isInfinity = true;
  }
  


  /**
  Returns the X-coordinate of the point unless the point is at infinity; the
  throws an IllegalStateException.
  */
  public BigInteger getX() {
	if (isInfinity) {
		throw new IllegalStateException();
	} else {
		return X;
	}
  }


  
  /**
  Returns the Y-coordinate of the point unless the point is at infinity; the
  throws an IllegalStateException.
  */
  public BigInteger getY() {
	if (isInfinity) {
		throw new IllegalStateException();
	} else {
		return Y;
	}
  }
  


  /**
  Returns <code>true</code>, if the point is at infinity. Otherwise 
  returns <code>false</code>.
  */
  public boolean isInfinity() {
	return isInfinity;
  }
  


  /**
    @doc
    Returns the ECDSA-specific key parameters. These parameters are
    never secret.
    
    @return the ECDSA-specific key parameters.
    
    @see ECDSAParams
  */
  public ECDSAParams getParams() {
	return curve;
  }
  
  
  
  /**
  Adds the parameter point P to this point.
  
  @return A new <code>ECPoint</code> representing the sum of this point and
  	the parameter point.
  */
  public ECPoint add(ECPoint P) {
	// Either point is infinity
	if (this.isInfinity()) {
		return P;
	} else if (P.isInfinity()) {
		return this;
	}		

	final BigInteger x0 = X;
	final BigInteger y0 = Y;
	final BigInteger x1 = P.getX();
	final BigInteger y1 = P.getY();
	BigInteger x2;
	BigInteger y2;
	

	//  this code after P1363
	if (x0.compareTo(x1) == 0) { 
		if (y0.compareTo(y1) != 0) {
			return new ECPointImpl(curve);	
		}
	
		if (y1.compareTo(BigInteger.ZERO) == 0) {
			return new ECPointImpl(curve);	
		}

		// Same point (=doubling)
		final BigInteger l = THREE.multiply(x1).multiply(x1).add(curve.getA()).multiply(TWO.multiply(y1).modInverse(curve.getP())).mod(curve.getP());
		x2 = l.multiply(l).subtract(x0).subtract(x1).mod(curve.getP());
		y2 = x1.subtract(x2).multiply(l).subtract(y1).mod(curve.getP());
		return new ECPointImpl(curve, x2, y2);
	}

	// Otherwise - different points
	final BigInteger l =  (y0.subtract(y1)).multiply(x0.subtract(x1).modInverse(curve.getP())).mod(curve.getP());
	x2 = l.multiply(l).subtract(x0).subtract(x1).mod(curve.getP());
	y2 = x1.subtract(x2).multiply(l).subtract(y1).mod(curve.getP());
	return new ECPointImpl(curve, x2, y2);
  }


  
  /**
  Subtracts the parameter point P from this point.
  
  @return A new <code>ECPoint</code> representing the difference of this point and
  	the parameter point.
  */
  public ECPoint subtract(ECPoint P) {
	return add(new ECPointImpl(curve, P.getX(), P.getY().negate()));
  }
  


  /**
  Multiplyes this point with the (scalar) parameter k.
  
  @return A new <code>ECPoint</code> representing the product of this point and
  	the parameter scalar value k.
  */
  public ECPoint multiply(BigInteger k) {

	// This algorithm is directly from ANSI X9.62, Annex D.3.2
	final BigInteger e = k.mod(curve.getN());
	if (e.compareTo(BigInteger.ZERO) == 0) {
		return new ECPointImpl(curve);
	}

	final BigInteger e3 = e.multiply(THREE);
	

	int r = e3.bitLength()-1;

	ECPointImpl R = this;

	
	for (int i = r-1; i >= 1; i--) {
		R = (ECPointImpl) R.add(R);

		if (e3.testBit(i) && !e.testBit(i)) {
			R = (ECPointImpl) R.add(this);
		}
		if (!e3.testBit(i) && e.testBit(i)) {
			R = (ECPointImpl) R.subtract(this);
		}

	}
	
	return R;
  }
  

  /**
	@doc
	Clones this point.
	
	@return A clone of this point.
  */
  public Object clone() {
	return new ECPointImpl(curve, X, Y);
  }
  

  /**
	@doc
	Verifyes, whether the point is on the curve or not.

	Normally not used , but can be used to verify points created elsewhere or to debug
	the implementation.
	
	@return <code>True</code>, if point lies on the curve, otherwise <code>false</code>.
  */
  public boolean verifyPoint() {
	if (curve.getP().compareTo(X) <= 0 || curve.getP().compareTo(Y) <= 0) {
		return false;
	}

	BigInteger u = Y.modPow(TWO, curve.getP());
	
	BigInteger v = X.pow(3);
	v = v.add(curve.getA().multiply(X));
	v = v.add(curve.getB());
	v = v.mod(curve.getP());

	return (u.compareTo(v) == 0);
  }
  
} // ECPointImpl
