/**
  Copyright (C) 2000 Helsinki University of Technology. All rights Reserved.
  XXX   %Z%%M% version %I% date:%G%
  Component:  BasicSignatureTest
  Package: fi.hut.tcm.tessa.ECTests
  File:    BasicSignatureTest.java
*/

package fi.hut.tcm.tessa.ECTests;
import java.security.*;
import java.io.*;
import fi.hut.tcm.tessa.ECImplementation.*;
import fi.hut.tcm.tessa.ECInterfaces.*;

/**
  BigSignatureTest
  
  This class tests the performance of the ECDSA Signature and demonstrates it's use.
*/
public class BigSignatureTest
  extends Object {

  /**
  A test string used for signatures. Can be changed at will.
  */
  private String message = "This is a test message.";
  
static byte[] tmpRes;


  /**
  Performs the signing and verification tests.
  */
  public static void main(String[] args) {
	// Number of times to loop through the program; used to get better measurements.
	int count = 1;
	// Check if new count was given as argument.
	if (args.length > 0) {
		count = (new Integer(args[0])).intValue();
	}
	
	// How strong a signature to create.
	int strength = 256;
	// Check if another strengt was given as argument.
	if (args.length > 1) {
		strength = (new Integer(args[1])).intValue();
	}

	// Install the ECDSA Provider (Necessary for using the ECDSA implementation).
	Security.addProvider(new ECProvider());

	KeyPairGenerator eg;
	Signature es;
	KeyPair ek;
	PrivateKey epri;
	PublicKey epub;
	long eTime;

	// Create a KeyPairGenerator object
	try {
		eg = KeyPairGenerator.getInstance("ECDSA");
	} catch (Exception e) {
		e.printStackTrace();
		// throw new RuntimeException("XXX");
		return;
	}

	// Create a Signature object
	try {
		es = Signature.getInstance("ECDSA");
	} catch (Exception e) {
		e.printStackTrace();
		// throw new RuntimeException("XXX");
		return;
	}
	eg.initialize(strength, null);

	System.out.println();
	System.out.println();
	System.out.println();

	// Create the first key pair. The system has to initialise, so this takes
	// longer than the other signatures.
	System.out.println("Timing FIRST key pair generation");
	eTime = timeGenerate(eg, 1);
	System.out.println("ECDSA (" + count + " key pairs): " + eTime + " ms");
	System.out.println();
	System.out.println();
	System.out.println();

	// Now, time a normal key pair generation
	System.out.println("Timing a normal key pair generation");
	eTime = timeGenerate(eg, count);
	System.out.println("ECDSA (" + count + " key pairs): " + eTime + " ms");
	ek = eg.generateKeyPair();
	System.out.println();
	System.out.println();
	System.out.println();

	epri = ek.getPrivate();
	epub = ek.getPublic();


	try {
		byte[] m = {127,32,46,98,0};
		byte[] p = {127,33,46,98,0};

		System.out.println("");
		System.out.println("Timing signing...");
		byte[] db;
		byte[] eb;
		// Now, time the signing.
		eTime = timeSigning(es, epri, m, count);
		eb = tmpRes;
		System.out.println("ECDSA (" + count + " signings): " + eTime + " ms");
		System.out.println();
		System.out.println();
		System.out.println();
		
		// And time the verification.
		System.out.println("Timing verify...");
		System.out.println("Starting to verify the CORRECT message... " +
							"(The result should be: TRUE)");
		eTime = timeVerify(es, epub, m, eb, count);
		System.out.println("ECDSA (" + count + " verifications): " + eTime + " ms");
		System.out.println();
		System.out.println();
		System.out.println();

		// Time the verification of a false message.
		System.out.println("Timing verify...");
		System.out.println("Starting to verify the MODIFIED message..." +
							"(The result should be: FALSE)");
		eTime = timeVerify(es, epub, p, eb, count);
		System.out.println("ECDSA (" + count + " verifications): " + eTime + " ms");
		System.out.println();
		System.out.println();
		System.out.println();

		// Test the key storage format.
		System.out.println("Testing the key storing and retrieval: ");
		System.out.println("First, signing with a converted key and verifying "+
							"with the normal key");
		byte[] eprib = epri.getEncoded();
		KeyFactory factory = KeyFactory.getInstance("ECDSA");
		ECDSAPrivateKeySpec privSpec = new ECDSAPrivateKeySpec(eprib);
		PrivateKey newPriv = factory.generatePrivate(privSpec);
		timeSigning(es, newPriv, m, 1);
		byte[] newSig = tmpRes;
		timeVerify(es, epub, m, newSig, 1);
		System.out.println();
		System.out.println();
		System.out.println();

		// Now, check that verification succeeds with stored key.
		System.out.println("Now, signing with the normal key and verifying with a converted key");
		byte[] epubb = epub.getEncoded();
		ECDSAPublicKeySpec pubSpec = new ECDSAPublicKeySpec(epubb);
		PublicKey newPub = factory.generatePublic(pubSpec);
		timeVerify(es, newPub, m, newSig, 1);
		
	}
	catch (Exception e) {
		e.printStackTrace();
	}

	System.out.println();
	System.out.println();
	System.out.println();
	System.out.println("Done.");
	return;
  }



  /*
	Used to test the key generation operation.
  */
  public static long timeGenerate(KeyPairGenerator g, int max) {
	long start, end;
	int i;

	start = System.currentTimeMillis();
	for (i = 0; i < max; i++) {
		g.generateKeyPair();
	}
	end = System.currentTimeMillis();
	return end - start;	
  }
		
  
  
  /*
	Used to time the signing operation.
  */
  public static long timeSigning(Signature s, PrivateKey pri, byte[] m, int max) {
	long start, end;
	int i;
	byte[] b = null;

	start = System.currentTimeMillis();
	for (i = 0; i < max; i++) {
		try {
			s.initSign(pri);
			s.update(m, 0, m.length);
			b = s.sign();
		} catch (Exception e) {
			e.printStackTrace();
			return 0;
		}
	}
	end = System.currentTimeMillis();
	tmpRes = b;
	return end - start;	
  }

  
  
  /*
	Used to time the verification operation.
  */
  public static long timeVerify(Signature s, PublicKey pub, byte[] m, byte[] b, int max) {
	long start, end;
	int i;
	boolean res = true;

	start = System.currentTimeMillis();
	for (i = 0; i < max; i++) {
		try {
			s.initVerify(pub);
			s.update(m, 0, m.length);
			res = s.verify(b);
		} catch (Exception e) {
			e.printStackTrace();
			return 0;
		}
	}
	end = System.currentTimeMillis();
	if (res) {
		System.out.println("The signature proved to be TRUE");
	} else {
		System.out.println("The signature proved to be FALSE");
	}
	return end - start;	
  }


  
} // basicSignatureTest

